@@ -149,9 +149,14 @@ void ExpressionVisitor::visitAssignmentExpression(AssignmentExpressionAst *node)
     }
 }
 
-void ExpressionVisitor::visitAssignmentExpressionEqual(AssignmentExpressionEqualAst *node)
+void ExpressionVisitor::visitBaseVariable(BaseVariableAst* node)
 {
-    DefaultVisitor::visitAssignmentExpressionEqual(node);
+    DefaultVisitor::visitBaseVariable(node);
+
+    if (node->offsetItemsSequence) {
+        // it's an array item but we don't support it really, so just assign type mixed and be done
+        m_result.setType(AbstractType::Ptr(new IntegralType(IntegralType::TypeMixed)));
+    }
 }
 
 void ExpressionVisitor::visitCompoundVariableWithSimpleIndirectReference(CompoundVariableWithSimpleIndirectReferenceAst *node)
@@ -181,8 +186,11 @@ void ExpressionVisitor::visitVarExpressionNewObject(VarExpressionNewObjectAst *n
 {
     DefaultVisitor::visitVarExpressionNewObject(node);
     if (node->className->identifier) {
-        Declaration* dec = findDeclarationImport(ClassDeclarationType, node->className->identifier);
-        usingDeclaration(node->className->identifier, dec);
+        const QualifiedIdentifier id = identifierForNamespace(node->className->identifier, m_editor);
+        Declaration* dec = findDeclarationImport(ClassDeclarationType, node->className->identifier,
+                                                 id);
+        usingDeclaration(node->className->identifier->namespaceNameSequence->back()->element, dec);
+        buildNamespaceUses(node->className->identifier, id);
         DUChainReadLocker lock(DUChain::lock());
         m_result.setDeclaration(dec);
     }
@@ -196,6 +204,75 @@ void ExpressionVisitor::visitVarExpressionNormal(VarExpressionNormalAst *node)
     }
 }
 
+void ExpressionVisitor::visitClosure(ClosureAst* node)
+{
+    FunctionType* closureType = new FunctionType;
+    m_result.setType(AbstractType::Ptr(new IntegralType(IntegralType::TypeVoid)));
+    visitInnerStatementList(node->functionBody);
+    closureType->setReturnType(m_result.type());
+
+    if (node->parameters->parametersSequence) {
+        const KDevPG::ListNode< ParameterAst* >* it = node->parameters->parametersSequence->front();
+        forever {
+            AbstractType::Ptr type;
+            if (it->element->parameterType) {
+                //don't use openTypeFromName as it uses cursor for findDeclarations
+                Declaration* decl = findDeclarationImport(ClassDeclarationType,
+                                                            it->element->parameterType,
+                                                            identifierForNamespace(it->element->parameterType, m_editor));
+                if (decl) {
+                    type = decl->abstractType();
+                }
+            } else if (it->element->arrayType != -1) {
+                type = AbstractType::Ptr(new IntegralType(IntegralType::TypeArray));
+            } else if (it->element->defaultValue) {
+                ExpressionVisitor v(m_editor);
+                it->element->defaultValue->ducontext = m_currentContext;
+                v.visitNode(it->element->defaultValue);
+                type = v.result().type();
+            }
+            if (!type) {
+                type = AbstractType::Ptr(new IntegralType(IntegralType::TypeMixed));
+            }
+
+            if ( it->element->isRef != -1 ) {
+                ReferenceType::Ptr p( new ReferenceType() );
+                p->setBaseType( type );
+
+                type = p.cast<AbstractType>();
+            }
+            closureType->addArgument(type);
+            if ( it->hasNext() ) {
+                it = it->next;
+            } else {
+                break;
+            }
+        }
+    }
+
+    if (node->lexicalVars) {
+        const KDevPG::ListNode< LexicalVarAst* >* it = node->lexicalVars->lexicalVarsSequence->front();
+        DUChainWriteLocker lock;
+        forever {
+            Declaration* found = 0;
+            foreach(Declaration* dec, m_currentContext->findDeclarations(identifierForNode(it->element->variable))) {
+                if (dec->kind() == Declaration::Instance) {
+                    found = dec;
+                    break;
+                }
+            }
+            usingDeclaration(it->element->variable, found);
+            if ( it->hasNext() ) {
+                it = it->next;
+            } else {
+                break;
+            }
+        }
+    }
+
+    m_result.setType(AbstractType::Ptr(closureType));
+}
+
 void ExpressionVisitor::visitFunctionCallParameterList( FunctionCallParameterListAst* node )
 {
     QList<Declaration*> decs = m_result.allDeclarations();
@@ -237,10 +314,12 @@ void ExpressionVisitor::visitFunctionCall(FunctionCallAst* node)
             //static function call foo::$bar()
         } else {
             //global function call foo();
-            Declaration* dec = findDeclarationImport(FunctionDeclarationType, node->stringFunctionNameOrClass);
+            const QualifiedIdentifier id = identifierForNamespace(node->stringFunctionNameOrClass, m_editor);
+            Declaration* dec = findDeclarationImport(FunctionDeclarationType, node->stringFunctionNameOrClass, id);
             ifDebug(kDebug() << "function call of" << (dec ? dec->toString() : QString("function not found"));)
             m_result.setDeclaration(dec);
-            usingDeclaration(node->stringFunctionNameOrClass, dec);
+            usingDeclaration(node->stringFunctionNameOrClass->namespaceNameSequence->back()->element, dec);
+            buildNamespaceUses(node->stringFunctionNameOrClass, id);
             if (dec) {
                 FunctionType::Ptr function = dec->type<FunctionType>();
                 if (function) {
@@ -270,40 +349,65 @@ DUContext* ExpressionVisitor::findClassContext(IdentifierAst* className)
     return context;
 }
 
+DUContext* ExpressionVisitor::findClassContext(NamespacedIdentifierAst* className)
+{
+    DUContext* context = 0;
+    const QualifiedIdentifier id = identifierForNamespace(className, m_editor);
+    static const QualifiedIdentifier staticQId("static");
+    if (id.count() == 1 && id == staticQId) {
+        return m_currentContext->parentContext();
+    }
+    Declaration* declaration = findDeclarationImport(ClassDeclarationType, className, id);
+    usingDeclaration(className->namespaceNameSequence->back()->element, declaration);
+    buildNamespaceUses(className, id);
+    if (declaration) {
+        DUChainReadLocker lock(DUChain::lock());
+        context = declaration->internalContext();
+        if (!context && m_currentContext->parentContext() && m_currentContext->parentContext()->localScopeIdentifier() == declaration->qualifiedIdentifier()) {
+            //className is currentClass (internalContext is not yet set)
+            context = m_currentContext->parentContext();
+        }
+    }
+    return context;
+}
+
 void ExpressionVisitor::visitConstantOrClassConst(ConstantOrClassConstAst *node)
 {
     DefaultVisitor::visitConstantOrClassConst(node);
 
-    if (node->className) {
+    if (node->classConstant) {
         //class constant Foo::BAR
-        DUContext* context = findClassContext(node->className);
+        DUContext* context = findClassContext(node->constant);
         if (context) {
             DUChainReadLocker lock(DUChain::lock());
-            m_result.setDeclarations(context->findDeclarations(identifierForNode(node->constant)));
+            m_result.setDeclarations(context->findDeclarations(Identifier(m_editor->parseSession()->symbol(node->classConstant))));
             lock.unlock();
             if (!m_result.allDeclarations().isEmpty()) {
-                usingDeclaration(node->constant, m_result.allDeclarations().last());
+                usingDeclaration(node->classConstant, m_result.allDeclarations().last());
             } else {
-                usingDeclaration(node->constant, 0);
+                usingDeclaration(node->classConstant, 0);
             }
         } else {
             m_result.setType(AbstractType::Ptr());
         }
-    } else if (node->constant) {
+    } else {
         QString str(stringForNode(node->constant).toLower());
         if (str == "true" || str == "false") {
             m_result.setType(AbstractType::Ptr(new IntegralType(IntegralType::TypeBoolean)));
         } else if (str == "null") {
             m_result.setType(AbstractType::Ptr(new IntegralType(IntegralType::TypeNull)));
         } else {
-            //constant (created with declare('foo', 'bar'))
-            Declaration* declaration = findDeclarationImport(ConstantDeclarationType, node->constant);
+            //constant (created with declare('foo', 'bar')) or const Foo = 1;
+            QualifiedIdentifier id = identifierForNamespace(node->constant, m_editor, true);
+            Declaration* declaration = findDeclarationImport(ConstantDeclarationType, node->constant, id);
             if (!declaration) {
+                ///TODO: is this really wanted?
                 //it could also be a global function call, without ()
-                declaration = findDeclarationImport(FunctionDeclarationType, node->constant);
+                declaration = findDeclarationImport(FunctionDeclarationType, node->constant, id);
             }
             m_result.setDeclaration(declaration);
-            usingDeclaration(node->constant, declaration);
+            usingDeclaration(node->constant->namespaceNameSequence->back()->element, declaration);
+            buildNamespaceUses(node->constant, id);
         }
     }
 }
@@ -366,11 +470,6 @@ void ExpressionVisitor::visitStaticScalar(StaticScalarAst *node)
     }
 }
 
-void ExpressionVisitor::visitEncaps(EncapsAst *node)
-{
-    DefaultVisitor::visitEncaps(node);
-}
-
 void ExpressionVisitor::visitEncapsVar(EncapsVarAst *node)
 {
     DefaultVisitor::visitEncapsVar(node);
@@ -517,6 +616,8 @@ void ExpressionVisitor::visitUnaryExpression(UnaryExpressionAst* node)
             type = IntegralType::TypeArray;
             break;
         case CastObject: {
+            /// Qualified identifier for 'stdclass'
+            static const KDevelop::QualifiedIdentifier stdclassQId("stdclass");
             DUChainReadLocker lock(DUChain::lock());
             m_result.setDeclarations(m_currentContext->findDeclarations(stdclassQId));
             break;
@@ -544,12 +645,12 @@ void ExpressionVisitor::visitAdditiveExpressionRest(AdditiveExpressionRestAst* n
     }
 }
 
-QString ExpressionVisitor::stringForNode(IdentifierAst* id)
+QString ExpressionVisitor::stringForNode(AstNode* id)
 {
     if (!id)
         return QString();
 
-    return m_editor->parseSession()->symbol(id->string);
+    return m_editor->parseSession()->symbol(id);
 }
 
 QualifiedIdentifier ExpressionVisitor::identifierForNode(IdentifierAst* id)
@@ -587,6 +688,19 @@ void ExpressionVisitor::setOffset(SimpleCursor offset)
     m_offset = offset;
 }
 
+void ExpressionVisitor::buildNamespaceUses(NamespacedIdentifierAst* namespaces, const QualifiedIdentifier& identifier)
+{
+    QualifiedIdentifier curId;
+    curId.setExplicitlyGlobal(identifier.explicitlyGlobal());
+    Q_ASSERT(identifier.count() == namespaces->namespaceNameSequence->count());
+    for ( int i = 0; i < identifier.count() - 1; ++i ) {
+        curId.push(identifier.at(i));
+        AstNode* node = namespaces->namespaceNameSequence->at(i)->element;
+        Declaration* dec = findDeclarationImport(NamespaceDeclarationType, node, curId);
+        usingDeclaration(node, dec);
+    }
+}
+
 Declaration* ExpressionVisitor::findDeclarationImport(DeclarationType declarationType, IdentifierAst* node)
 {
     // methods and class names are case insensitive
@@ -596,12 +710,17 @@ Declaration* ExpressionVisitor::findDeclarationImport(DeclarationType declaratio
     } else {
         id = identifierForNode(node);
     }
-    return findDeclarationImportHelper(m_currentContext, id, declarationType, node, m_editor);
+    return findDeclarationImport(declarationType, node, id);
 }
 
 Declaration* ExpressionVisitor::findDeclarationImport(DeclarationType declarationType, VariableIdentifierAst* node)
 {
-    return findDeclarationImportHelper(m_currentContext, identifierForNode(node), declarationType, node, m_editor);
+    return findDeclarationImport(declarationType, node, identifierForNode(node));
+}
+
+Declaration* ExpressionVisitor::findDeclarationImport(DeclarationType declarationType, AstNode* node, const KDevelop::QualifiedIdentifier& identifier)
+{
+    return findDeclarationImportHelper(m_currentContext, identifier, declarationType, node, m_editor);
 }
 
 }
